<script setup lang="ts">
import { gantt, Task } from 'dhtmlx-gantt'
import { PropType } from 'vue'

defineOptions({
  name: 'XGantt',
})

const ganttContainerRef = ref<HTMLElement | string>('')

const props = defineProps({
  tasks: {
    type: Object as PropType<{ data: any[]; links: any[] }>,
    default: () => ({ data: [], links: [] }),
  },
})

const UNASSIGNED_ID = 5
const WORK_DAY = 8

const shouldHighlightTask = (task: Task) => {
  // const store = gantt.$resourcesStore
  const store: any = gantt.getDatastore(gantt.config.resource_store)
  const taskResource = task[gantt.config.resource_property]
  const selectedResource = store.getSelectedId()
  if (taskResource == selectedResource || store.isChildOf(taskResource, selectedResource)) {
    return true
  }
}

const getAllocatedValue = (tasks: Array<Task>, resource: any) => {
  let result = 0
  tasks.forEach((item) => {
    const assignments = gantt.getResourceAssignments(resource.id, item.id)
    assignments.forEach((assignment) => {
      result += Number(assignment.value)
    })
  })
  return result
}

const resourceTemplates = {
  grid_row_class: (start: Date, end: Date, resource: any) => {
    const css = []
    if (resource.$level === 0) {
      css.push('folder_row')
      css.push('group_row')
    }
    if (shouldHighlightResource(resource)) {
      css.push('highlighted_resource')
    }
    return css.join(' ')
  },
  task_row_class: (start: Date, end: Date, resource: any) => {
    const css = []
    if (shouldHighlightResource(resource)) {
      css.push('highlighted_resource')
    }
    if (resource.$level === 0) {
      css.push('group_row')
    }
    return css.join(' ')
  },
}

const shouldHighlightResource = (resource: any) => {
  const selectedTaskId = gantt.getState().selected_task
  if (gantt.isTaskExists(selectedTaskId)) {
    const selectedTask = gantt.getTask(selectedTaskId)
    const selectedResource = selectedTask[gantt.config.resource_property]

    if (resource.id == selectedResource) {
      return true
    } else if (gantt.$resourcesStore.isChildOf(selectedResource, resource.id)) {
      return true
    }
  }
  return false
}

const getResourceAssignments = (resourceId: string | number) => {
  let assignments: Array<any>
  const store = gantt.getDatastore(gantt.config.resource_store)
  const resource: any = store.getItem(resourceId)

  if (resource.$level === 0) {
    assignments = []
    store.getChildren(resourceId).forEach((childId: any) => {
      assignments = assignments.concat(gantt.getResourceAssignments(childId))
    })
  } else if (resource.$level === 1) {
    assignments = gantt.getResourceAssignments(resourceId)
  } else {
    assignments = gantt.getResourceAssignments(resource.$resource_id, resource.$task_id)
  }
  return assignments
}

const toggleGroups = () => {
  gantt.$groupMode = !gantt.$groupMode
  if (gantt.$groupMode) {
    // input.value = 'show gantt view'

    const groups = gantt.$resourcesStore.getItems().map((item: any) => {
      const group = gantt.copy(item)
      group.group_id = group.id
      group.id = gantt.uid()
      return group
    })

    gantt.groupBy({
      groups: groups,
      relation_property: gantt.config.resource_property,
      group_id: 'group_id',
      group_text: 'text',
      delimiter: ', ',
      default_group_label: 'Not Assigned',
    })
  } else {
    // input.value = 'show resource view'
    gantt.groupBy(false)
  }
}

// const cap = {};
// const getCapacity = (date, resource) => {
//   console.log("==>", resource, [0, 1, 2, 3][Math.floor(Math.random() * 100) % 4])
//   if (resource.$level !== 1) {
//     return -1;
//   }
//
//   const val = date.valueOf();
//   if (!cap[val + resource.id]) {
//     cap[val + resource.id] = [0, 1, 2, 3][Math.floor(Math.random() * 100) % 4];
//   }
//   return cap[val + resource.id] * WORK_DAY;
// };

gantt.plugins({
  grouping: true,
  auto_scheduling: true,
})

gantt.config.auto_scheduling = true
gantt.config.auto_scheduling_strict = true
gantt.config.work_time = true
gantt.config.resource_store = 'resource'
gantt.config.resource_property = 'owner'
gantt.config.resource_render_empty_cells = true
gantt.config.order_branch = true
gantt.config.open_tree_initially = true
gantt.config.scale_height = 50

gantt.templates.grid_row_class = (start: Date, end: Date, task: Task) => {
  const css = []

  if (gantt.hasChild(task.id)) {
    css.push('folder_row')
  }

  if (task.$virtual) {
    css.push('group_row')
  }

  if (shouldHighlightTask(task)) {
    css.push('highlighted_resource')
  }

  return css.join(' ')
}

gantt.templates.task_row_class = (start: Date, end: Date, task: Task) => {
  if (shouldHighlightTask(task)) {
    return 'highlighted_resource'
  }
  return ''
}

gantt.templates.timeline_cell_class = (task: any, date: Date) => {
  if (!gantt.isWorkTime({ date: date, task: task })) {
    return 'week_end'
  }
  return ''
}

gantt.templates.histogram_cell_class = (start_date: Date, end_date: Date, resource: any, tasks: Array<Task>) => {
  if (resource.$level === 1) {
    if (getAllocatedValue(tasks, resource) > WORK_DAY) {
      return 'column_overload'
    }
  } else if (resource.$level === 2) {
    return 'resource_task_cell'
  }
}

gantt.templates.histogram_cell_label = (start_date: Date, end_date: Date, resource: any, tasks: Array<Task>) => {
  if (tasks.length && resource.$level === 1) {
    return `${getAllocatedValue(tasks, resource)}/${WORK_DAY}`
  } else if (resource.$level === 0) {
    return ''
  } else if (resource.$level === 2) {
    if (gantt.isWorkTime({ date: start_date, task: gantt.getTask(resource.$task_id) })) {
      if (start_date.valueOf() < resource.end_date.valueOf() && end_date.valueOf() > resource.start_date.valueOf()) {
        const assignment = gantt.getResourceAssignments(resource.$resource_id, resource.$task_id)[0]
        return assignment.value
      } else {
        return '–'
      }
    }
  }
  return '–'
}

gantt.templates.histogram_cell_allocated = (start_date: Date, end_date: Date, resource: any, tasks: Array<Task>) => {
  return getAllocatedValue(tasks, resource)
}

gantt.templates.histogram_cell_capacity = (start_date: Date, end_date: Date, resource: any, tasks: Array<Task>) => {
  if (!gantt.isWorkTime(start_date)) {
    return 0
  }

  if (resource.$level !== 1) {
    return -1
  }
  return getAllocatedValue(tasks, resource)
  // return WORK_DAY
  // return getCapacity(start_date, resource);
}

gantt.locale.labels.section_owner = 'Owner'
gantt.config.lightbox.sections = [
  { name: 'description', height: 70, map_to: 'text', type: 'textarea', focus: true },
  {
    name: 'owner',
    type: 'resources',
    map_to: 'owner',
    options: gantt.serverList('people'),
    default_value: WORK_DAY,
    unassigned_value: UNASSIGNED_ID,
  },
  { name: 'time', type: 'duration', map_to: 'auto' },
]

const resourceConfig = {
  scale_height: 30,
  row_height: 45,
  scales: [{ unit: 'day', step: 1, date: '%d %M' }],
  columns: [
    {
      name: 'name',
      label: 'Name',
      tree: true,
      width: 200,
      template: (resource: any) => {
        return resource.text
      },
      resize: true,
    },
    {
      name: 'progress',
      label: 'Complete',
      align: 'center',
      template: (resource: any) => {
        // const store = gantt.getDatastore(gantt.config.resource_store);
        let totalToDo = 0
        let totalDone = 0

        let completion = 0
        if (resource.$level == 2) {
          completion = resource.progress * 100
        } else {
          const assignments = getResourceAssignments(resource.id)
          assignments.forEach((assignment) => {
            const task: Task = gantt.getTask(assignment.task_id)
            if (task.duration !== undefined) {
              totalToDo += task.duration
              totalDone += task.duration * (task.progress || 0)
            }
          })

          if (totalToDo) {
            completion = (totalDone / totalToDo) * 100
          }
        }

        return `${Math.floor(completion)}%`
      },
      resize: true,
    },
    {
      name: 'workload',
      label: 'Workload',
      align: 'center',
      template: (resource: any) => {
        let totalDuration = 0
        if (resource.$level == 2) {
          const assignments = gantt.getResourceAssignments(resource.$resource_id, resource.$task_id)
          if (assignments.length) {
            totalDuration = resource.duration * assignments[0].value
          }
        } else {
          const assignments = getResourceAssignments(resource.id)
          assignments.forEach((assignment: any) => {
            const task = gantt.getTask(assignment.task_id)
            if (task.duration !== undefined) {
              totalDuration += Number(assignment.value) * task.duration
            }
          })
        }
        return `${totalDuration || 0}h`
      },
      resize: true,
    },
    {
      name: 'capacity',
      label: 'Capacity',
      align: 'center',
      template: (resource: any) => {
        if (resource.$level == 2) {
          return `${resource.duration * WORK_DAY}h`
        }
        const store = gantt.getDatastore(gantt.config.resource_store)
        const n = resource.$level === 0 ? store.getChildren(resource.id).length : 1
        const state = gantt.getState()
        return `${gantt.calculateDuration(state.min_date, state.max_date) * n * WORK_DAY}h`
      },
    },
  ],
}

gantt.config.scales = [
  { unit: 'month', step: 1, format: '%F, %Y' },
  { unit: 'day', step: 1, format: '%d %M' },
]

gantt.config.columns = [
  { name: 'text', tree: true, width: 150, resize: true },
  { name: 'start_date', align: 'center', width: 80, resize: true },
  {
    name: 'owner',
    align: 'center',
    width: 80,
    label: 'Owner',
    template: (task) => {
      if (task.type == gantt.config.types.project) {
        return ''
      }
      const store = gantt.getDatastore('resource')
      const assignments = task[gantt.config.resource_property]

      if (!assignments || !assignments.length) {
        return 'Unassigned'
      }

      if (assignments.length == 1) {
        return (store.getItem(assignments[0].resource_id) as any)?.text
      }

      let result = ''
      assignments.forEach((assignment: any) => {
        const owner: any = store.getItem(assignment.resource_id)
        if (!owner) return
        result += `<div class='owner-label' title='${owner.text}'>${owner.text.substr(0, 1)}</div>`
      })

      return result
    },
    resize: true,
  },
  { name: 'duration', width: 60, align: 'center', resize: true },
  { name: 'add', width: 44 },
]

gantt.config.layout = {
  css: 'gantt_container',
  rows: [
    {
      gravity: 2,
      cols: [
        { view: 'grid', group: 'grids', scrollY: 'scrollVer' },
        { resizer: true, width: 1 },
        { view: 'timeline', scrollX: 'scrollHor', scrollY: 'scrollVer' },
        { view: 'scrollbar', id: 'scrollVer', group: 'vertical' },
      ],
    },
    { resizer: true, width: 1, next: 'resources' },
    {
      height: 35,
      cols: [
        {
          html: "<label>Resource<select class='resource-select'></select>",
          css: 'resource-select-panel',
          group: 'grids',
        },
        { resizer: true, width: 1 },
        { html: '' },
      ],
    },
    {
      gravity: 1,
      id: 'resources',
      config: resourceConfig,
      templates: resourceTemplates,
      cols: [
        { view: 'resourceGrid', group: 'grids', scrollY: 'resourceVScroll' },
        { resizer: true, width: 1 },
        { view: 'resourceHistogram', capacity: WORK_DAY, scrollX: 'scrollHor', scrollY: 'resourceVScroll' },
        { view: 'scrollbar', id: 'resourceVScroll', group: 'vertical' },
      ],
    },
    { view: 'scrollbar', id: 'scrollHor' },
  ],
}

gantt.attachEvent('onTaskLoading', (task) => {
  const ownerValue = task[gantt.config.resource_property]

  if (!task.$virtual && (!ownerValue || !Array.isArray(ownerValue) || !ownerValue.length)) {
    task[gantt.config.resource_property] = [{ resource_id: 5, value: 0 }] //'Unassigned' group
  }
  return true
})

const resourcesStore = gantt.createDatastore({
  name: gantt.config.resource_store,
  type: 'treeDatastore',
  fetchTasks: false,
  initItem: (item: any) => {
    item.parent = item.parent || gantt.config.root_id
    item[gantt.config.resource_property] = item.parent
    item.open = true
    return item
  },
})

// gantt.config.resources = {
//   resource_store: {
//     type: 'treeDatastore',
//     fetchTasks: false,
//     initItem: (item: any) => {
//       item.parent = item.parent || gantt.config.root_id
//       item[gantt.config.resource_property] = item.parent
//       item.open = true
//       return item
//     },
//   },
// }

resourcesStore.attachEvent('onAfterSelect', () => {
  gantt.refreshData()
})
//
resourcesStore.attachEvent('onParse', () => {
  const people: any[] = []

  resourcesStore.eachItem((res: any) => {
    if (res.$level === 1) {
      const copy = gantt.copy(res)
      copy.key = res.id
      copy.label = res.text
      copy.unit = 'hours'
      people.push(copy)
    }
  })
  gantt.updateCollection('people', people)
})
//
resourcesStore.parse([
  { id: 1, text: 'QA', parent: null },
  { id: 2, text: 'Development', parent: null },
  { id: 3, text: 'Sales', parent: null },
  { id: 4, text: 'Other', parent: null },
  { id: 5, text: 'Unassigned', parent: 4, default: true },
  { id: 6, text: 'John', parent: 1 },
  { id: 7, text: 'Mike', parent: 2 },
  { id: 8, text: 'Anna', parent: 2 },
  { id: 9, text: 'Bill', parent: 3 },
  { id: 10, text: 'Floe', parent: 3 },
])

onMounted(() => {
  gantt.init(ganttContainerRef.value)
  gantt.parse(props.tasks)

  // gantt.config.date_format = '%Y-%m-%d';
  //
  // if (ganttContainerRef.value){
  //   gantt.plugins({grouping: true, auto_scheduling: true})
  //   gantt.init(ganttContainerRef.value);
  //   gantt.parse(props.tasks);
  // }
})
</script>

<template>
  <div>
    <div class="gantt_control">
      <input id="default" type="button" value="Show Resource view" @click="toggleGroups" />
    </div>
    <div ref="ganttContainerRef" class="demo xgantt"></div>
  </div>
</template>

<style>
@import 'dhtmlx-gantt/codebase/dhtmlxgantt.css';
.demo.xgantt {
  width: 100%;
  padding: 0;
  border: none;
  height: calc(100vh - 270px);
}

.gantt_grid_scale .gantt_grid_head_cell,
.gantt_task .gantt_task_scale .gantt_scale_cell {
  font-weight: bold;
  font-size: 14px;
  color: rgba(0, 0, 0, 0.7);
}

.folder_row {
  font-weight: bold;
}

.highlighted_resource,
.highlighted_resource.odd {
  background-color: rgba(255, 251, 224, 0.6);
}

.gantt_task_cell.week_end {
  background-color: #e8e8e87d;
}

.gantt_task_row.gantt_selected .gantt_task_cell.week_end {
  background-color: #e8e8e87d !important;
}

.group_row,
.group_row.odd,
.gantt_task_row.group_row {
  background-color: rgba(232, 232, 232, 0.6);
}

.owner-label {
  width: 20px;
  height: 20px;
  line-height: 20px;
  font-size: 12px;
  display: inline-block;
  border: 1px solid #cccccc;
  border-radius: 25px;
  background: #e6e6e6;
  color: #6f6f6f;
  margin: 0 3px;
  font-weight: bold;
}

.resource-select-panel .gantt_layout_content {
  line-height: 35px;
  text-align: right;
  padding-right: 15px;
  overflow: hidden !important;
}
.resource-select {
  padding: 3px;
  margin-left: 13px;
}

.column_overload .gantt_histogram_fill {
  background-color: #ffa9a9;
}

.resource_task_cell .gantt_histogram_label {
  font-weight: normal;
}
</style>
